/* options.c

Copyright (C) 1999-2003 Tom Gilbert.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to
deal in the Software without restriction, including without limitation the
rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies of the Software and its documentation and acknowledgment shall be
given in the documentation and software packages that this Software was
used.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/

#include "feh.h"
#include "filelist.h"
#include "options.h"

static void check_options(void);
static void feh_parse_option_array(int argc, char **argv);
static void feh_parse_environment_options(void);
static void feh_check_theme_options(int arg, char **argv);
static void feh_parse_options_from_string(char *opts);
static void feh_load_options_for_theme(char *theme);
static void show_usage(void);
static void show_version(void);
static char *theme;

fehoptions opt;

void init_parse_options(int argc, char **argv)
{
	/* TODO: sort these to match declaration of __fehoptions */

	/* For setting the command hint on X windows */
	cmdargc = argc;
	cmdargv = argv;

	/* Set default options */
	memset(&opt, 0, sizeof(fehoptions));
	opt.display = 1;
	opt.aspect = 1;
	opt.slideshow_delay = 0.0;
	opt.thumb_w = 60;
	opt.thumb_h = 60;
	opt.thumb_redraw = 10;
	opt.menu_font = estrdup(DEFAULT_MENU_FONT);
	opt.font = NULL;
	opt.image_bg = estrdup("default");
	opt.menu_bg = estrdup(PREFIX "/share/feh/images/menubg_default.png");
	opt.menu_style = estrdup(PREFIX "/share/feh/fonts/menu.style");

	opt.reload_button = 0;
	opt.pan_button = 1;
	opt.zoom_button = 2;
	opt.menu_button = 3;
	opt.menu_ctrl_mask = 0;
	opt.prev_button = 4;
	opt.next_button = 5;

	opt.draw_actions = 0;

	opt.rotate_button = 2;
	opt.no_rotate_ctrl_mask = 0;
	opt.blur_button = 1;
	opt.no_blur_ctrl_mask = 0;

	opt.start_list_at = NULL;
	opt.jump_on_resort = 1;

	opt.builtin_http = 0;

	opt.xinerama = 0;
	opt.screen_clip = 1;
#ifdef HAVE_LIBXINERAMA
	/* if we're using xinerama, then enable it by default */
	opt.xinerama = 1;
#endif				/* HAVE_LIBXINERAMA */

	D(("About to parse env options (if any)\n"));
	/* Check for and parse any options in FEH_OPTIONS */
	feh_parse_environment_options();

	D(("About to parse commandline options\n"));
	/* Parse the cmdline args */
	feh_parse_option_array(argc, argv);

	D(("About to check for theme configuration\n"));
	feh_check_theme_options(argc, argv);

	/* If we have a filelist to read, do it now */
	if (opt.filelistfile) {
		/* joining two reverse-sorted lists in this manner works nicely for us
		   here, as files specified on the commandline end up at the *end* of
		   the combined filelist, in the specified order. */
		D(("About to load filelist from file\n"));
		filelist = gib_list_cat(filelist, feh_read_filelist(opt.filelistfile));
	}

	D(("Options parsed\n"));

	if (opt.bgmode)
		return;

	filelist_len = gib_list_length(filelist);
	if (!filelist_len)
		show_mini_usage();

	check_options();

	feh_prepare_filelist();
	return;
}

static void feh_check_theme_options(int arg, char **argv)
{
	if (!theme) {
		/* This prevents screw up when running src/feh or ./feh */
		char *pos = strrchr(argv[0], '/');

		if (pos)
			theme = estrdup(pos + 1);
		else
			theme = estrdup(argv[0]);
	}
	D(("Theme name is %s\n", theme));

	feh_load_options_for_theme(theme);

	free(theme);
	return;
	arg = 0;
}

static void feh_load_options_for_theme(char *theme)
{
	FILE *fp = NULL;
	char *home = getenv("HOME");
	char *rcpath = NULL;
	char *confbase = getenv("XDG_CONFIG_HOME");
	char s[1024], s1[1024], s2[1024];
	int cont = 0;
	int bspos;

	if (!home)
		eprintf("You have no HOME, cannot read themes");

	if (confbase)
		rcpath = estrjoin("/", confbase, "feh/themes", NULL);
	else
		rcpath = estrjoin("/", home, ".config/feh/themes", NULL);

	fp = fopen(rcpath, "r");

	free(rcpath);

	if (!fp && ((fp = fopen("/etc/feh/themes", "r")) == NULL))
		return;

	/* Oooh. We have an options file :) */
	for (; fgets(s, sizeof(s), fp);) {
		s1[0] = '\0';
		s2[0] = '\0';

		if (cont) {
			sscanf(s, " %[^\n]\n", (char *) &s2);
			if (!*s2)
				break;
			D(("Got continued options %s\n", s2));
		} else {
			sscanf(s, "%s %[^\n]\n", (char *) &s1, (char *) &s2);
			if (!(*s1) || (!*s2) || (*s1 == '\n') || (*s1 == '#')) {
				cont = 0;
				continue;
			}
			D(("Got theme/options pair %s/%s\n", s1, s2));
		}

		if (!strcmp(s1, theme) || cont) {

			bspos = strlen(s2)-1;

			if (s2[bspos] == '\\') {
				D(("Continued line\n"));
				s2[bspos] = '\0';
				cont = 1;
				/* A trailing whitespace confuses the option parser */
				if (bspos && (s2[bspos-1] == ' '))
					s2[bspos-1] = '\0';
			} else
				cont = 0;

			D(("A match. Using options %s\n", s2));
			feh_parse_options_from_string(s2);

			if (!cont)
				break;
		}
	}
	fclose(fp);
	return;
}

static void feh_parse_environment_options(void)
{
	char *opts;

	if ((opts = getenv("FEH_OPTIONS")) == NULL)
		return;

	weprintf
	    ("The FEH_OPTIONS configuration method is depreciated and will soon die.\n"
	     "Use the feh/themes configuration file instead.");

	/* We definitely have some options to parse */
	feh_parse_options_from_string(opts);
	return;
}

/* FIXME This function is a crufty bitch ;) */
static void feh_parse_options_from_string(char *opts)
{
	char **list = NULL;
	int num = 0;
	char *s;
	char *t;
	char last = 0;
	int inquote = 0;
	int i = 0;

	/* So we don't reinvent the wheel (not again, anyway), we use the
	   getopt_long function to do this parsing as well. This means it has to
	   look like the real argv ;) */

	list = malloc(sizeof(char *));

	list[num++] = estrdup(PACKAGE);

	for (s = opts, t = opts;; t++) {
		if ((*t == ' ') && !(inquote)) {
			*t = '\0';
			num++;
			list = erealloc(list, sizeof(char *) * num);

			list[num - 1] = feh_string_normalize(s);
			s = t + 1;
		} else if (*t == '\0') {
			num++;
			list = erealloc(list, sizeof(char *) * num);

			list[num - 1] = feh_string_normalize(s);
			break;
		} else if (*t == '\"' && last != '\\')
			inquote = !(inquote);
		last = *t;
	}

	feh_parse_option_array(num, list);

	for (i = 0; i < num; i++)
		if (list[i])
			free(list[i]);
	if (list)
		free(list);
	return;
}

char *feh_string_normalize(char *str)
{
	char ret[4096];
	char *s;
	int i = 0;
	char last = 0;

	D(("normalizing %s\n", str));
	ret[0] = '\0';

	for (s = str;; s++) {
		if (*s == '\0')
			break;
		else if ((*s == '\"') && (last == '\\'))
			ret[i++] = '\"';
		else if ((*s == '\"') && (last == 0));
		else if ((*s == ' ') && (last == '\\'))
			ret[i++] = ' ';
		else
			ret[i++] = *s;

		last = *s;
	}
	if (i && ret[i - 1] == '\"')
		ret[i - 1] = '\0';
	else
		ret[i] = '\0';
	D(("normalized to %s\n", ret));

	return(estrdup(ret));
}

static void feh_parse_option_array(int argc, char **argv)

{
	static char stropts[] =
		"a:A:b:B:cC:dD:e:E:f:Fg:GhH:iIj:J:kK:lL:mM:nNo:O:pPqQrR:sS:tT:uUvVwW:xXy:YzZ"
		"0:1:2:4:5:8:9:.@:^:~:):|:+:";

	/* (*name, has_arg, *flag, val) See: struct option in getopts.h */
	static struct option lopts[] = {
		{"help"          , 0, 0, 'h'},
		{"version"       , 0, 0, 'v'},
		{"montage"       , 0, 0, 'm'},
		{"collage"       , 0, 0, 'c'},
		{"index"         , 0, 0, 'i'},
		{"fullindex"     , 0, 0, 'I'},
		{"verbose"       , 0, 0, 'V'},
		{"borderless"    , 0, 0, 'x'},
		{"keep-http"     , 0, 0, 'k'},
		{"stretch"       , 0, 0, 's'},
		{"multiwindow"   , 0, 0, 'w'},
		{"recursive"     , 0, 0, 'r'},
		{"randomize"     , 0, 0, 'z'},
		{"list"          , 0, 0, 'l'},
		{"quiet"         , 0, 0, 'q'},
		{"loadable"      , 0, 0, 'U'},
		{"unloadable"    , 0, 0, 'u'},
		{"no-menus"      , 0, 0, 'N'},
		{"full-screen"   , 0, 0, 'F'}, /* deprecated */
		{"fullscreen"    , 0, 0, 'F'},
		{"auto-zoom"     , 0, 0, 'Z'},
		{"ignore-aspect" , 0, 0, 'X'},
		{"draw-filename" , 0, 0, 'd'},
		{"preload"       , 0, 0, 'p'},
		{"reverse"       , 0, 0, 'n'},
		{"thumbnails"    , 0, 0, 't'},
		{"builtin"       , 0, 0, 'Q'},
		{"scale-down"    , 0, 0, '.'},
		{"no-jump-on-resort", 0, 0, 220},
		{"hide-pointer"  , 0, 0, 'Y'},
		{"draw-actions"  , 0, 0, 'G'},
		{"cache-thumbnails", 0, 0, 'P'},
		{"cycle-once"    , 0, 0, 224},
		{"no-xinerama"   , 0, 0, 225},
		{"no-rotate-ctrl-mask", 0, 0, 226},
		{"no-blur-ctrl-mask", 0, 0, 227},
		{"menu-ctrl-mask", 0, 0, 228},

		{"output"        , 1, 0, 'o'},
		{"output-only"   , 1, 0, 'O'},
		{"action"        , 1, 0, 'A'},
		{"limit-width"   , 1, 0, 'W'},
		{"limit-height"  , 1, 0, 'H'},
		{"reload"        , 1, 0, 'R'},
		{"alpha"         , 1, 0, 'a'},
		{"sort"          , 1, 0, 'S'},
		{"theme"         , 1, 0, 'T'},
		{"filelist"      , 1, 0, 'f'},
		{"customlist"    , 1, 0, 'L'},
		{"geometry"      , 1, 0, 'g'},
		{"menu-font"     , 1, 0, 'M'},
		{"thumb-width"   , 1, 0, 'y'},
		{"thumb-height"  , 1, 0, 'E'},
		{"slideshow-delay",1, 0, 'D'},
		{"font"          , 1, 0, 'e'},
		{"title-font"    , 1, 0, '@'},
		{"title"         , 1, 0, '^'},
		{"thumb-title"   , 1, 0, '~'},
		{"bg"            , 1, 0, 'b'},
		{"fontpath"      , 1, 0, 'C'},
		{"menu-bg"       , 1, 0, ')'},
		{"image-bg"      , 1, 0, 'B'},
		{"reload-button" , 1, 0, '0'},
		{"pan-button"    , 1, 0, '1'},
		{"zoom-button"   , 1, 0, '2'},
		{"menu-button"   , 1, 0, '3'},
		{"prev-button"   , 1, 0, '4'},
		{"next-button"   , 1, 0, '5'},
		{"rotate-button" , 1, 0, '8'},
		{"blur-button"   , 1, 0, '9'},
		{"start-at"      , 1, 0, '|'},
		{"debug"         , 0, 0, '+'},
		{"output-dir"    , 1, 0, 'j'},
		{"bg-tile"       , 1, 0, 200},
		{"bg-center"     , 1, 0, 201},
		{"bg-scale"      , 1, 0, 202},
		{"menu-style"    , 1, 0, 204},
		{"zoom"          , 1, 0, 205},
		{"no-screen-clip", 0, 0, 206},
		{"caption-path"  , 1, 0, 'K'},
		{"action1"       , 1, 0, 209},
		{"action2"       , 1, 0, 210},
		{"action3"       , 1, 0, 211},
		{"action4"       , 1, 0, 212},
		{"action5"       , 1, 0, 213},
		{"action6"       , 1, 0, 214},
		{"action7"       , 1, 0, 215},
		{"action8"       , 1, 0, 216},
		{"action9"       , 1, 0, 217},
		{"bg-fill"       , 1, 0, 218},
		{"bg-max"        , 1, 0, 219},
		{"index-name"    , 1, 0, 230},
		{"index-size"    , 1, 0, 231},
		{"index-dim"     , 1, 0, 232},
		{"thumb-redraw"  , 1, 0, 'J'},
		{"info"          , 1, 0, 234},

		{0, 0, 0, 0}
	};
	int optch = 0, cmdx = 0;
	int i = 0;

	/* Now to pass some optionarinos */
	while ((optch = getopt_long(argc, argv, stropts, lopts, &cmdx)) != EOF) {
		D(("Got option, getopt calls it %d, or %c\n", optch, optch));
		switch (optch) {
		case 0:
			break;
		case 'h':
			show_usage();
			break;
		case 'v':
			show_version();
			break;
		case 'm':
			opt.index = 1;
			opt.index_show_name = 0;
			opt.index_show_size = 0;
			opt.index_show_dim = 0;
			break;
		case 'c':
			opt.collage = 1;
			break;
		case 'i':
			opt.index = 1;
			opt.index_show_name = 1;
			opt.index_show_size = 0;
			opt.index_show_dim = 0;
			break;
		case '.':
			opt.scale_down = 1;
			break;
		case 'I':
			opt.index = 1;
			opt.index_show_name = 1;
			opt.index_show_size = 1;
			opt.index_show_dim = 1;
			break;
		case 'l':
			opt.list = 1;
			opt.display = 0;
			break;
		case 'Q':
			opt.builtin_http = 1;
			break;
		case 'L':
			opt.customlist = estrdup(optarg);
			opt.display = 0;
			break;
		case 'M':
			free(opt.menu_font);
			opt.menu_font = estrdup(optarg);
			break;
		case '+':
			opt.debug = 1;
			break;
		case 'n':
			opt.reverse = 1;
			break;
		case 'g':
			opt.geom_flags = XParseGeometry(optarg, &opt.geom_x, &opt.geom_y, &opt.geom_w, &opt.geom_h);
			break;
		case 'N':
			opt.no_menus = 1;
			break;
		case 'V':
			opt.verbose = 1;
			break;
		case 'q':
			opt.quiet = 1;
			break;
		case 'x':
			opt.borderless = 1;
			break;
		case 'k':
			opt.keep_http = 1;
			break;
		case 's':
			opt.stretch = 1;
			break;
		case 'w':
			opt.multiwindow = 1;
			break;
		case 'r':
			opt.recursive = 1;
			break;
		case 'z':
			opt.randomize = 1;
			break;
		case 'd':
			opt.draw_filename = 1;
			break;
		case 'F':
			opt.full_screen = 1;
			break;
		case 'Z':
			opt.auto_zoom = 1;
			break;
		case 'U':
			opt.loadables = 1;
			opt.display = 0;
			break;
		case 'u':
			opt.unloadables = 1;
			opt.display = 0;
			break;
		case 'p':
			opt.preload = 1;
			break;
		case 'X':
			opt.aspect = 0;
			break;
		case 'S':
			if (!strcasecmp(optarg, "name"))
				opt.sort = SORT_NAME;
			else if (!strcasecmp(optarg, "filename"))
				opt.sort = SORT_FILENAME;
			else if (!strcasecmp(optarg, "width"))
				opt.sort = SORT_WIDTH;
			else if (!strcasecmp(optarg, "height"))
				opt.sort = SORT_HEIGHT;
			else if (!strcasecmp(optarg, "pixels"))
				opt.sort = SORT_PIXELS;
			else if (!strcasecmp(optarg, "size"))
				opt.sort = SORT_SIZE;
			else if (!strcasecmp(optarg, "format"))
				opt.sort = SORT_FORMAT;
			else {
				weprintf("Unrecognised sort mode \"%s\". Defaulting to "
						"sort by filename", optarg);
				opt.sort = SORT_FILENAME;
			}
			break;
		case 'o':
			opt.output = 1;
			opt.output_file = estrdup(optarg);
			break;
		case 'O':
			opt.output = 1;
			opt.output_file = estrdup(optarg);
			opt.display = 0;
			break;
		case 'T':
			theme = estrdup(optarg);
			break;
		case 'C':
			D(("adding fontpath %s\n", optarg));
			imlib_add_path_to_font_path(optarg);
			break;
		case 'e':
			opt.font = estrdup(optarg);
			break;
		case '@':
			opt.title_font = estrdup(optarg);
			break;
		case '^':
			opt.title = estrdup(optarg);
			break;
		case '~':
			opt.thumb_title = estrdup(optarg);
			break;
		case 'b':
			opt.bg = 1;
			opt.bg_file = estrdup(optarg);
			break;
		case 'A':
			opt.actions[0] = estrdup(optarg);
			break;
		case 'W':
			opt.limit_w = atoi(optarg);
			break;
		case 'H':
			opt.limit_h = atoi(optarg);
			break;
		case 'y':
			opt.thumb_w = atoi(optarg);
			break;
		case 'E':
			opt.thumb_h = atoi(optarg);
			break;
		case ')':
			free(opt.menu_bg);
			opt.menu_bg = estrdup(optarg);
			break;
		case 'B':
			free(opt.image_bg);
			opt.image_bg = estrdup(optarg);
			break;
		case 'D':
			opt.slideshow_delay = atof(optarg);
			if (opt.slideshow_delay < 0.0) {
				opt.slideshow_delay *= (-1);
				opt.paused = 1;
			}
			break;
		case 'R':
			opt.reload = atoi(optarg);
			break;
		case 'a':
			opt.alpha = 1;
			opt.alpha_level = 255 - atoi(optarg);
			break;
		case 'f':
			opt.filelistfile = estrdup(optarg);
			break;
		case '0':
			opt.reload_button = atoi(optarg);
			break;
		case '1':
			opt.pan_button = atoi(optarg);
			break;
		case '2':
			opt.zoom_button = atoi(optarg);
			break;
		case '3':
			opt.menu_button = atoi(optarg);
			break;
		case '4':
			opt.prev_button = atoi(optarg);
			break;
		case '5':
			opt.next_button = atoi(optarg);
			break;
		case '8':
			opt.rotate_button = atoi(optarg);
			break;
		case '9':
			opt.blur_button = atoi(optarg);
			break;
		case '|':
			opt.start_list_at = estrdup(optarg);
			break;
		case 't':
			opt.thumbs = 1;
			opt.index_show_name = 1;
			opt.index_show_size = 0;
			opt.index_show_dim = 0;
			break;
		case 'j':
			opt.output_dir = estrdup(optarg);
			break;
		case 200:
			opt.bgmode = BG_MODE_TILE;
			opt.output_file = estrdup(optarg);
			break;
		case 201:
			opt.bgmode = BG_MODE_CENTER;
			opt.output_file = estrdup(optarg);
			break;
		case 202:
			opt.bgmode = BG_MODE_SCALE;
			opt.output_file = estrdup(optarg);
			break;
		case 218:
			opt.bgmode = BG_MODE_FILL;
			opt.output_file = estrdup(optarg);
			break;
		case 219:
			opt.bgmode = BG_MODE_MAX;
			opt.output_file = estrdup(optarg);
			break;
		case 204:
			free(opt.menu_style);
			opt.menu_style = estrdup(optarg);
			break;
		case 205:
			opt.default_zoom = atoi(optarg);
			break;
		case 206:
			opt.screen_clip = 0;
			break;
		case 'K':
			opt.caption_path = estrdup(optarg);
			break;
		case 209:
			opt.actions[1] = estrdup(optarg);
			break;
		case 210:
			opt.actions[2] = estrdup(optarg);
			break;
		case 211:
			opt.actions[3] = estrdup(optarg);
			break;
		case 212:
			opt.actions[4] = estrdup(optarg);
			break;
		case 213:
			opt.actions[5] = estrdup(optarg);
			break;
		case 214:
			opt.actions[6] = estrdup(optarg);
			break;
		case 215:
			opt.actions[7] = estrdup(optarg);
			break;
		case 216:
			opt.actions[8] = estrdup(optarg);
			break;
		case 217:
			opt.actions[9] = estrdup(optarg);
			break;
		case 220:
			opt.jump_on_resort = 0;
			break;
		case 'Y':
			opt.hide_pointer = 1;
			break;
		case 'G':
			opt.draw_actions = 1;
			break;
		case 'P':
			opt.cache_thumbnails = 1;
			break;
		case 224:
			opt.cycle_once = 1;
			break;
		case 225:
			opt.xinerama = 0;
			break;
		case 226:
			opt.no_rotate_ctrl_mask = 1;
			break;
		case 227:
			opt.no_blur_ctrl_mask = 1;
			break;
		case 228:
			opt.menu_ctrl_mask = 1;
			break;
		case 230:
			opt.index_show_name = atoi(optarg);
			break;
		case 231:
			opt.index_show_size = atoi(optarg);
			break;
		case 232:
			opt.index_show_dim = atoi(optarg);
			break;
		case 'J':
			opt.thumb_redraw = atoi(optarg);
			break;
		case 234:
			opt.info_cmd = estrdup(optarg);
			break;
		default:
			break;
		}
	}

	/* Now the leftovers, which must be files */
	if (optind < argc) {
		while (optind < argc) {
			/* If recursive is NOT set, but the only argument is a directory
			   name, we grab all the files in there, but not subdirs */
			add_file_to_filelist_recursively(argv[optind++], FILELIST_FIRST);
		}
	}

	for (i = 0; i < 10; i++) {
		if (opt.actions[i] && !opt.hold_actions[i] && (opt.actions[i][0] == ';')) {
			opt.hold_actions[i] = 1;
			opt.actions[i] = &opt.actions[i][1];
		}
	}

	/* So that we can safely be called again */
	optind = 1;
	return;
}

static void check_options(void)
{
	if ((opt.index + opt.collage) > 1) {
		weprintf("you can't use collage mode and index mode together.\n"
				"   I'm going with index");
		opt.collage = 0;
	}

	if (opt.full_screen && opt.multiwindow) {
		weprintf("you shouldn't combine multiwindow mode with full-screen mode,\n"
				"   Multiwindow mode has been disabled.");
		opt.multiwindow = 0;
	}

	if (opt.list && (opt.multiwindow || opt.index || opt.collage)) {
		weprintf("list mode can't be combined with other processing modes,\n"
				"   list mode disabled.");
		opt.list = 0;
	}

	if (opt.sort && opt.randomize) {
		weprintf("You cant sort AND randomize the filelist...\n"
				"randomize mode has been unset\n");
		opt.randomize = 0;
	}

	if (opt.loadables && opt.unloadables) {
		weprintf("You cant show loadables AND unloadables...\n"
				"you might as well use ls ;)\n"
				"loadables only will be shown\n");
		opt.unloadables = 0;
	}

	if (opt.thumb_title && (!opt.thumbs)) {
		weprintf("Doesn't make sense to set thumbnail title when not in\n"
				"thumbnail mode.\n");
		free(opt.thumb_title);
		opt.thumb_title = NULL;
	}

	return;
}

static void show_version(void)
{
	printf(PACKAGE " version " VERSION "\n");
	exit(0);
}

void show_mini_usage(void)
{
	fprintf(stderr, PACKAGE " - No loadable images specified.\n"
			"Use " PACKAGE " --help for detailed usage information\n");
	exit(1);
}

static void show_usage(void)
{
	fputs(
#include "help.inc"
	, stdout);
	exit(0);
}
